def LiveLoadReduction(AT, KLL=1):
    """
    ASCE 7 live load reduction factor

    AT - tributary area (ft^2)
    KLL - component factor (default = 1)
    """

    KLLAT = KLL*AT
    
    if KLLAT > 400.0:
        return max(0.4,0.25 + 15/KLLAT**0.5)
    else:  
        return 1.0

def VelocityPressure(z, exposure):
    """
    ASCE 7 wind velocity pressure

    z - height above ground (ft)
    exposure - category ('B', 'C', or 'D')
    """

    if exposure not in ['B','C','D']:
        return None
    
    if exposure == 'B':
        zg = 1200.0
        alpha = 7.0
    if exposure == 'C':
        zg = 900.0
        alpha = 9.5
    if exposure == 'D':
        zg = 700.0
        alpha = 11.5

    if z < 0.0:
        return 0.0
        
    if z < 15.0:
        return 2.01*(15.0/zg)**(2.0/alpha)
    else:
        return 2.01*(z/zg)**(2.0/alpha)

def SiteFactorShortPeriod(soil, Ss):
    """
    ASCE 7 soil site factor for short period spectral acceleration

    soil - classification ('A', 'B', 'C', 'D', or 'E')
    Ss - short period spectral acceleration (g)
    """
    
    if soil == 'A':
        return 0.8
    if soil == 'B':
        return 0.9
    if soil == 'C':
        if Ss <= 0.5:
            return 1.3
        else:
            return 1.2
    if soil == 'D':
        if Ss <= 0.25:
            return 1.6
        if Ss <= 0.5:
            return 1.4
        if Ss <= 0.75:
            return 1.2
        if Ss <= 1.0:
            return 1.1
        return 1.0
    if soil == 'E':
        if Ss <= 0.25:
            return 2.4
        if Ss <= 0.5:
            return 1.7
        if Ss <= 0.75:
            return 1.3
        return None
    return None

def SiteFactorLongPeriod(soil, S1):
    """
    ASCE 7 soil site factor for long period spectral acceleration

    soil - classification ('A', 'B', 'C', 'D', or 'E')
    S1 - long period spectral acceleration (g)
    """
    
    if soil == 'A':
        return 0.8
    if soil == 'B':
        return 0.8
    if soil == 'C':
        if S1 <= 0.5:
            return 1.5
        else:
            return 1.4
    if soil == 'D':
        if S1 <= 0.1:
            return 2.4
        if S1 <= 0.2:
            return 2.2
        if S1 <= 0.3:
            return 2.0
        if S1 <= 0.4:
            return 1.9
        if S1 <= 0.5:
            return 1.8        
        return 1.7
    if soil == 'E':
        if S1 <= 0.1:
            return 4.2
        return None
    return None



import math

def CreepModelCode2010(fcm,h0,t0,t,**kwargs):
    """Creep according to Model Code 2010
    fcm - mean 28-day cylinder compressive strength (MPa)
    h0 - nominal thickness (mm), h0 = 2*A/O where A = section area, O = section circumference
    t0 - loading age (days)
    t - time at analysis (days)
    """

    # Default values
    RCA = kwargs.get('RCA', 0)    
    aggregate = kwargs.get('aggregate', 'quartzite')
    cement = kwargs.get('cement', '42.5R')
    RH = kwargs.get('RH', 75.0)    
    Temp = kwargs.get('Temp', 25)
    epscb2 = kwargs.get('epscb2', 1)
    epscd2 = kwargs.get('epscd2', 1)
    epsccRCA = kwargs.get('epsccRCA', 1)    
    
    if aggregate == 'sandstone':
        alphaNA = 0.7
    if aggregate == 'limestone':
        alphaNA = 0.9
    if aggregate == 'quartzite':
        alphaNA = 1
    if aggregate == 'basalt':
        alphaNA = 1.2

    if cement in ['42.5R','52.5N','52.5R']:
        s = 0.2
        alpha = 1
    if cement in ['32.5R','42.5N']:
        s = 0.25
        alpha = 0
    if cement in ['32.5N']:
        s = 0.38
        alpha = -1
        
    # 28-day modulus of elasticity
    Ecm = 21500*alphaNA*(fcm/10)**0.3333*(1-0.3*RCA/100)

    # Compressive strength at loading age
    fct0 = fcm*math.exp(s*(1-(28/t0)**0.5))

    # Modulus of elasticity at loading age
    Ect0 = Ecm*(math.exp(s*(1-(28/t0)**0.5)))**0.5

    # Tensile strength at loading age
    fctmt0 = 0.3*(fct0-8)**0.6667

    # Loading age adjusted for cement type
    t0Tadj = t0
    
    #
    alphafcm = (35/fcm)**0.5

    #
    betah = min(1.5*h0+250*alphafcm,1500*alphafcm)

    tt0 = t-t0

    betabcfcm = 1.8/fcm**0.7
    print(betabcfcm)
    betabctt0 = math.log((30/t0Tadj+0.035)**2 * tt0/epscb2 + 1)

    betadcfcm = 412/fcm**1.4

    betaRH = (1-RH/100)/((0.1*h0/100)**0.3333)

    betadct0 = 1/(0.1 + t0Tadj**0.2)

    gammat0 = 1/(2.3 + 3.5/t0Tadj**0.5)

    betadctt0 = (tt0/(betah*epscd2+tt0))**gammat0

    phibctt0 = betabctt0*betabcfcm*epsccRCA

    phidctt0 = betadctt0*betadct0*betaRH*betadcfcm*epsccRCA

    phitt0 = phibctt0 + phidctt0

    data = {}

    data['phiba'] = betabcfcm*epsccRCA
    data['phibb'] = epscb2
    data['phida'] = betadcfcm*betaRH*epsccRCA
    data['phidb'] = betah*epscd2
    data['cem'] = alpha

    return data
    
def ShrinkageModelCode2010(fcm,h0,ts,t,**kwargs):
    """Shrinkage according to Model Code 2010
    fcm - mean 28-day cylinder compressive strength (MPa)
    h0 - nominal thickness (mm), h0 = 2*A/O where A = section area, O = section circumference
    ts - time at start of drying (days)
    t - time at analysis (days)
    """

    # Default values
    RCA = kwargs.get('RCA', 0)
    cement = kwargs.get('cement', '42.5R')
    RH = kwargs.get('RH', 75.0)
    Temp = kwargs.get('Temp', 25)    
    epscbs2 = kwargs.get('epscbs2', 1)
    epscds2 = kwargs.get('epscds2', 1)
    
    if cement in ['42.5R','52.5N','52.5R']:
        alphabs = 600
        alphads1 = 6
        alphads2 = 0.012
    if cement in ['32.5R','42.5N']:
        alphabs = 700
        alphads1 = 4
        alphads2 = 0.012
    if cement in ['32.5N']:
        alphabs = 800
        alphads1 = 3
        alphads2 = 0.013

    epscbs0 = -alphabs*((fcm/10)/(6+fcm/10))**2.5
    epscds0 = (220+110*alphads1)*math.exp(-alphads2*fcm)
    betas1 = min((35/fcm)**0.1,1)
    epscsRCA = max((RCA/fcm)**0.3,1)

    # Time at which shrinkage is calculated
    tts = t-ts
    
    betaas = 1-math.exp(-0.2*epscbs2*tts**0.5)
    betaRH = -1.55*(1-(RH/100)**3) if RH < 99*betas1 else 0.25
    betads = (tts/(0.035*h0**2*epscds2 + tts))**0.5

    epscbs = epscbs0*betaas*epscsRCA
    epscds = epscds0*betaRH*betads*epscsRCA
    epscs = epscbs + epscds

    data = {}

    data['epsba'] = epscsRCA*epscbs0*1e-6
    data['epsbb'] = epscbs2
    data['epsda'] = epscsRCA*epscds0*betaRH*1e-6
    data['epsdb'] = 0.035*h0**2*epscds2

    return data
    
def ShrinkageACI209R(fcm,VoverS,ts,t,**kwargs):
    """Shrinkage according to ACI209R-92
    fcm - mean 28-day cylinder compressive strength (MPa)
    VoverS - average thickness (mm), ratio of section area to section circumference
    ts - time at start of drying (days)
    t - time at analysis (days)
    RH - relative humidity (%)
    epsshu0 - ultimate shrinkage strain
    """

    # Default values
    epsshu0 = kwargs.get('epsshu0', 780e-6)

    RH = kwargs.get('RH', 75.0)
    slump = kwargs.get('slump', None)
    fineAgg = kwargs.get('fineAgg', None)
    c = kwargs.get('c', None)
    air = kwargs.get('air', None)            

    #
    #
    #
    
    # Parameter in shrinkage time evolution function
    f = 26*math.exp(0.0142*VoverS)

    # Correction for member size
    gammashVS = 1.2*math.exp(-0.00472*VoverS)
    
    # Correction for slump
    gammashs = 1
    if slump is not None:
        gammashs = 0.89+0.00161*slump

    # Correction for fine aggregate content
    gammashpsi = 1
    if fineAgg is not None:
        gammashpsi = 0.3+0.014*fineAgg if psi < 0.5 else 0.9+0.002*fineAgg
        
    # Correction for cement content
    gammashc = 1
    if c is not None:
        gammashc = 0.75+0.00061*c

    # Correction for air content
    gammasha = 1
    if air is not None:
        gammasha = max(1,0.95+0.008*air)

    # Time at which shrinkage is calculated
    tts = t-ts

    # Correction for curing time
    gammashtc = 1.202-0.2337*math.log10(ts)

    # Correction for relative humidity
    gammashRH = 1.4-1.02*RH/100 if RH < 80 else 3-3*RH/100

    # Global correction for ultimate shrinkage strain
    gammash = gammashRH*gammashtc*gammashVS*gammashs*gammashpsi*gammashc

    # Ultimate shrinkage strain
    epsshu = epsshu0*gammash
    betattc = tts/(f+tts)
    epsshtts = epsshu*betattc

    data = {}
    
    data['epsshu'] = -epsshu
    data['psish'] = f

    return data

def CreepACI209R(fcm,VoverS,t0,t,**kwargs):
    """Creep according to ACI209R-92
    fcm - mean 28-day cylinder compressive strength (MPa)
    VoverS - average thickness (mm), ratio of section area to section circumference
    t0 - loading age (days)
    t - time at analysis (days)
    RH - relative humidity (%)
    psi - exponent in creep time evolution function
    phiu0 - ultimate creep
    """

    # Default values
    psi = kwargs.get('psi', 1.0)
    phiu0 = kwargs.get('phiu0', 2.35)

    RH = kwargs.get('RH', 75.0)
    slump = kwargs.get('slump', None)
    fineAgg = kwargs.get('fineAgg', None)
    c = kwargs.get('c', None)
    air = kwargs.get('air', None)            

    #
    #
    #
    
    # Parameter in creep time evolution function
    d = 26*math.exp(0.0142*VoverS)

    # Correction for member size
    gammacVS = 2.0/3*(1+1.13*math.exp(-0.0213*VoverS))
    
    # Correction for slump
    gammacs = 1
    if slump is not None:
        gammacs = 0.82 + 0.00264*slump

    # Correction for fine aggregate content
    gammacpsi = 1
    if fineAgg is not None:
        gammacpsi = 0.88+0.0024*fineAgg
        
    # Correction for air content
    gammaca = 1
    if air is not None:
        gammaca = max(1,0.46+0.09*air)


    # Time at which creep is calculated
    tt0 = t-t0

    # Correction for loading age
    gammact0 = 1.25/(t0**0.118)
    
    # Correction for relative humidity
    gammacRH = 1.27-0.67*RH/100 if RH > 40 else 1.0

    # Global correction of ultimate creep coefficient
    gammac = gammacRH*gammact0*gammaca*gammacpsi*gammacs*gammacVS
    
    phiu = gammac*phiu0

    betatt0 = (tt0**psi)/(d + tt0**psi)
    phitt0 = betatt0*phiu

    data = {}
    
    data['phiu'] = phiu
    data['psicr1'] = psi
    data['psicr2'] = d

    return data

if __name__ == '__main__':

    import openseespy.opensees as ops

    fc = 28.3
    ft = 3
    Ec = 30000
    beta = 0.4
    ts = 7
    t0 = 7
    tcast = 4
    
    shrinkage = ShrinkageACI209R(45.7,39,ts,1007)
    creep = CreepACI209R(45.7,39,t0,1007)

    ops.uniaxialMaterial('TDConcrete',1,-fc,ft,Ec,beta,ts,
                         shrinkage['epsshu'],shrinkage['psish'],t0,
                         creep['phiu'],creep['psicr1'],creep['psicr2'],tcast)

    
    shrinkage = ShrinkageModelCode2010(45.7,77.8,7,357)
    shrinkage = ShrinkageModelCode2010(fc,150,14,10000)
    #print(shrinkage)

    creep = CreepModelCode2010(45.7,77.8,7,357)
    creep = CreepModelCode2010(fc,150,28,10000)
    print(creep)    
    



def isint(value):
    try:
        int(value)
        return True
    except ValueError:
        return False

def ConvertToOpenSees(infile, outfile):
    """
    Convert a beaverFrame input file to an OpenSeesPy script

    infile - name of beaverFrame input file to read
    outfile - name of OpenSeesPy script to write
    """
    
    inputt = open(infile,'r')
    output = open(outfile,'w')
    output2 = open('bennyFrame.py','w')

    output.write('import openseespy.opensees as ops\n')
    output.write('import numpy as np\n')
    output.write('ops.wipe()\n')

    Nnode = 0
    Nmat = 0
    Nsec = 0
    Ntransf = 0
    Nele = 0
    Nloadcase = 0

    springTag = 1

    for line in inputt:

        info = line.split()
        if len(info) < 1:
            continue

        if info[0] == 'NDM':
            ndm = int(info[1])
            if ndm < 1 or ndm > 3:
                print(f'ERROR in NDM={ndm}')
            ndf = 6
            if ndm == 2:
                ndf = 3
            output.write(f"ops.model('basic','-ndm',{ndm},'-ndf',{ndf})\n")

        if info[0] == 'NODES':
            Nnode = int(info[1])
            output2.write(f'NODES = np.zeros(({Nnode},{ndm}))\n')
            output2.write(f'SUPPORTS = np.zeros(({Nnode},{ndf}))\n')
            output2.write(f'NODALLOADS = np.zeros(({Nnode},{ndf}))\n')
            output2.write(f'SUPPORTDISPS = np.zeros(({Nnode},{ndf}))\n')                
            for i in range(Nnode):
                nodeInfo = inputt.readline().split()

                tag = int(nodeInfo[0])
                if ndm == 1:
                    X = float(nodeInfo[1])
                    output.write(f"ops.node({tag},{X})\n")
                    if isint(nodeInfo[2]):
                        DX = int(nodeInfo[2])
                    else:
                        DX = 0
                    output.write(f"ops.fix({tag},{DX})\n")            
                if ndm == 2:
                    X = float(nodeInfo[1])
                    Y = float(nodeInfo[2])
                    output.write(f"ops.node({tag},{X},{Y})\n")            
                    DX = float(nodeInfo[3])
                    DY = float(nodeInfo[4])
                    RZ = float(nodeInfo[5])
                    output.write(f"ops.fix({tag},{DX},{DY},{RZ})\n")                        
                if ndm == 3:
                    X = float(nodeInfo[1])
                    Y = float(nodeInfo[2])
                    Z = float(nodeInfo[3])
                    output.write(f"ops.node({tag},{X},{Y},{Z})\n")
                    output2.write(f'NODES[{tag-1},:]=[{X},{Y},{Z}]\n')
                    
                    DX = float(nodeInfo[4])
                    DY = float(nodeInfo[5])
                    DZ = float(nodeInfo[6])
                    RX = float(nodeInfo[7])
                    RY = float(nodeInfo[8])
                    RZ = float(nodeInfo[9])
                    isSupport = False
                    flexibleSupport = False
                    supportDOFs = []
                    for idof,dof in enumerate([DX,DY,DZ,RX,RY,RZ]):
                        if dof < 0:
                            isSupport = True
                            supportDOFs.append(1)
                        if dof == 0:
                            supportDOFs.append(0)
                        if dof > 0:
                            isSupport = True
                            flexibleSupport = True
                            output.write(f"ops.uniaxialMaterial('Elastic',{springTag},{dof})\n")
                            supportDOFs.append(-springTag)
                            springTag += 1
                    if isSupport and not flexibleSupport:
                        output.write(f"ops.fix({tag},{DX},{DY},{DZ},{RX},{RY},{RZ})\n")
                        output2.write(f'SUPPORTS[{tag-1},:]=[{DX},{DY},{DZ},{RX},{RY},{RZ}]\n')
                    if flexibleSupport:
                        output.write(f"ops.node({-tag},{X},{Y},{Z})\n")
                        output.write(f"ops.fix({-tag},1,1,1,1,1,1)\n")
                        #output.write(f"ops.element('zeroLength',{-tag},{-tag},{tag},'-mats','-dirs')\n")
        if info[0] == 'MATERIALS':
            Nmat = int(info[1])
            matData = [0]*(2*Nmat)
            for i in range(Nmat):
                matInfo = inputt.readline().split()
                tag = int(matInfo[0])
                E = float(matInfo[1])
                G = float(matInfo[2])
                matData[2*i    ] = E
                matData[2*i + 1] = G

        if info[0] == 'SECTIONS':
            Nsec = int(info[1])
            output2.write(f'SECTIONS = np.zeros(({Nsec},6))\n')                
            secData = [0]*(6*Nsec)
            for i in range(Nsec):
                secInfo = inputt.readline().split()
                tag = int(secInfo[0])
                matTag = int(secInfo[1])
                matIndex = matTag - 1
                E = matData[2*matIndex    ]
                G = matData[2*matIndex + 1]
                A   = float(secInfo[2])
                Iz  = float(secInfo[3])
                Avy = float(secInfo[4])
                secData[6*i    ] = A
                secData[6*i + 1] = Iz
                secData[6*i + 2] = Avy
                if ndm == 2:
                    output2.write(f'SECTIONS[{tag-1},:]=[{E},{A},{Iz}]\n')
                    if Avy <= 0.0 or Avz <= 0.0:
                        output.write(f"ops.section('Elastic',{tag},{E},{A},{Iz})\n")
                    else:
                        output.write(f"ops.section('Elastic',{tag},{E},{A},{Iz},{G},{Avy/A})\n")
                if ndm == 3:
                    Iy  = float(secInfo[5])
                    Avz = float(secInfo[6])
                    J   = float(secInfo[7])
                    secData[6*i + 3] = Iy
                    secData[6*i + 4] = Avz
                    secData[6*i + 5] = J
                    output2.write(f'SECTIONS[{tag-1},:]=[{E},{A},{Iz},{Iy},{G},{J}]\n')
                    if Avy <= 0.0 or Avz <= 0.0:
                        output.write(f"ops.section('Elastic',{tag},{E},{A},{Iz},{Iy},{G},{J})\n")
                    else:
                        output.write(f"ops.section('Elastic',{tag},{E},{A},{Iz},{Iy},{G},{J},{Avy/A},{Avz/A})\n")

        if info[0] == 'TRANSFORMATIONS':
            Ntransf = int(info[1])
            transfData = [0]*(4*Ntransf)
            for i in range(Ntransf):
                transfInfo = inputt.readline().split()
                tag = int(transfInfo[0])
                if ndm == 2:
                    transfData[4*i + 3] = float(transfInfo[1])                                                
                if ndm == 3:
                    transfData[4*i    ] = float(transfInfo[1])
                    transfData[4*i + 1] = float(transfInfo[2])
                    transfData[4*i + 2] = float(transfInfo[3])
                    transfData[4*i + 3] = float(transfInfo[4])                                

        if info[0] == 'ELEMENTS':
            Nele = int(info[1])
            output2.write(f'ELEMS = np.zeros(({Nele},6))\n')
            output2.write(f'MEMBERLOADS = np.zeros(({Nele},3))\n')                        
            for i in range(Nele):
                eleInfo = inputt.readline().split()
                tag = int(eleInfo[0])
                ndI = int(eleInfo[1])
                ndJ = int(eleInfo[2])                        
                transfTag = int(eleInfo[3])
                transfIndex = transfTag - 1
                output.write(f'x = np.array(ops.nodeCoord({ndJ})) - np.array(ops.nodeCoord({ndI}))\n')
                yx = transfData[4*transfIndex]
                yy = transfData[4*transfIndex + 1]
                yz = transfData[4*transfIndex + 2]
                output2.write(f'ELEMS[{tag-1},:]=[{ndI},{ndJ},{tag-1},{yx},{yy},{yz}]\n')
                output.write(f'yp = np.array([{yx},{yy},{yz}])\n')
                output.write(f'z = np.cross(x,yp)\n')
                #output.write(f'y = np.cross(z,x)\n')
                #output.write(f'z = np.cross(x,y)\n')            
                transfType = 'Linear'
                if transfData[4*transfIndex + 3] > 0.0:
                    transfType = 'PDelta'
                if ndm == 2:
                    output.write(f"ops.geomTransf('{transfType}',{tag})\n")
                else:
                    output.write(f"ops.geomTransf('{transfType}',{tag},*z)\n")
                secTag = int(eleInfo[4])
                kzI = float(eleInfo[5])
                kzJ = float(eleInfo[6])
                kyI = -1
                kyJ = -1
                if ndm == 3:
                    kyI = float(eleInfo[7])
                    kyJ = float(eleInfo[8])

                # An element with rigid,infinitesimal joints
                if kzI < 0 and kzJ < 0 and kyI < 0 and kyJ < 0:
                    output.write(f"ops.beamIntegration('NewtonCotes',{tag},{secTag},7)\n")
                    output.write(f"ops.element('forceBeamColumn',{tag},{ndI},{ndJ},{tag},{tag})\n")
                    continue

                relz = 0
                if kzI == 0:
                    relz = 1
                if kzJ == 0:
                    relz = 2
                if kzI == 0 and kzJ == 0:
                    relz = 3

                rely = 0
                if kyI == 0:
                    rely = 1
                if kyJ == 0:
                    rely = 2
                if kyI == 0 and kyJ == 0:
                    rely = 3

                # An element with moment releases at one or both ends
                if relz > 0 or rely > 0:
                    if ndm == 2:
                        output.write(f"ops.element('elasticBeamColumn',{tag},{ndI},{ndJ},{secTag},{tag})\n")
                        output.write(f"ops.parameter(1,'element',{tag},'release')\n")
                        output.write(f"ops.updateParameter(1,{relz})\n")
                        output.write(f"ops.remove('parameter',1)\n")
                    if ndm == 3:
                        output.write(f"ops.element('elasticBeamColumn',{tag},{ndI},{ndJ},{secTag},{tag},'-releasez',{relz},'-releasey',{rely})\n")
                    continue

                output.write(f"ops.uniaxialMaterial('Elastic',{springTag},{kzI})\n")
                springTag += 1
                output.write(f"ops.uniaxialMaterial('Elastic',{springTag},{kzJ})\n")
                springTag += 1
                if kyI < 0.0:
                    kyI = 1e4*kzI
                output.write(f"ops.uniaxialMaterial('Elastic',{springTag},{kyI})\n")
                springTag += 1
                if kyJ < 0.0:
                    kyJ = 1e4*kzJ            
                output.write(f"ops.uniaxialMaterial('Elastic',{springTag},{kyJ})\n")
                springTag += 1
                
                secIndex = secTag - 1
                A   = secData[6*secIndex    ]
                Iz  = secData[6*secIndex + 1]
                Avy = secData[6*secIndex + 2]
                Iy  = secData[6*secIndex + 3]
                Avz = secData[6*secIndex + 4]
                J   = secData[6*secIndex + 5]

                # An element with rotational springs at both ends
                if ndm == 2:
                    output.write(f"ops.element('componentElement',{tag},{ndI},{ndJ},{A},{E},{Iz},{tag},{springTag-4},{springTag-3})\n")
                if ndm == 3:
                    output.write(f"ops.element('componentElement',{tag},{ndI},{ndJ},{A},{E},{G},{J},{Iy},{Iz},{tag},{springTag-4},{springTag-3},{springTag-2},{springTag-1})\n")


        if info[0] == 'LOADCASES':
            Nloadcase += 1
            output.write("ops.constraints('Transformation')\n")
            output.write("ops.analysis('Static')\n")
            output.write("ops.timeSeries('Constant',1)\n")        
            output.write("output = open('beaverFrame.out','w')\n")
                
        if info[0] == 'LOADCASE':
            tag = int(info[1])
            output.write(f"ops.pattern('Plain',{tag},1)\n")

            # Support displacements
            sdInfo = inputt.readline().split()
            Nsupportdisp = int(sdInfo[1])
            for i in range(Nsupportdisp):
                sdInfo = inputt.readline().split()
                nd = int(sdInfo[0])
                D = [0]*ndf
                for j in range(ndf):
                    D[j] = float(sdInfo[j+1])
                output.write(f'ops.sp({nd},*{D})\n')            

            # Point loads
            plInfo = inputt.readline().split()
            Npointload = int(plInfo[1])
            for i in range(Npointload):
                plInfo = inputt.readline().split()
                nd = int(plInfo[0])
                P = [0]*ndf
                for j in range(ndf):
                    P[j] = float(plInfo[j+1])
                output.write(f'ops.load({nd},*{P})\n')

            # Member loads
            mlInfo = inputt.readline().split()
            Nmemberload = int(mlInfo[1])
            for i in range(Nmemberload):
                mlInfo = inputt.readline().split()
                ele = int(mlInfo[0])
                if ndm == 2:
                    pass
                if ndm == 3:
                    wX = float(mlInfo[1])
                    wY = float(mlInfo[2])
                    wZ = float(mlInfo[3])
                    output.write(f"x = ops.eleResponse({ele},'xaxis')\n")
                    output.write(f"y = ops.eleResponse({ele},'yaxis')\n")
                    output.write(f"z = ops.eleResponse({ele},'zaxis')\n")                
                    output.write(f"wx = np.dot(x,[{wX},{wY},{wZ}])\n")
                    output.write(f"wy = np.dot(y,[{wX},{wY},{wZ}])\n")
                    output.write(f"wz = np.dot(z,[{wX},{wY},{wZ}])\n")                
                    output.write(f"ops.eleLoad('-ele',{ele},'-type','beamUniform',wy,wz,wx)\n")

            # First clear out old mass
            output.write(f'for nd in ops.getNodeTags():\n')
            P = [0]*ndf
            output.write(f'\tops.mass(nd,*{P})\n')

            # Member masses
            mmInfo = inputt.readline().split()
            Nmembermass = int(mmInfo[1])
            for i in range(Nmembermass):
                mmInfo = inputt.readline().split()
                ele = int(mmInfo[0])
                output.write(f"ndI,ndJ = ops.eleNodes({ele})\n")
                output.write(f"XYZI = ops.nodeCoord(ndI)\n")
                output.write(f"XYZJ = ops.nodeCoord(ndJ)\n")
                output.write(f"Lele = np.linalg.norm(np.subtract(XYZJ,XYZI))\n")
                mX = float(mmInfo[1])
                mY = float(mmInfo[2])
                mZ = 0
                if ndm == 2:
                    mZ = mY
                if ndm == 3:
                    mZ = float(mmInfo[3])
                P = [0]*ndf
                for j in range(ndm):
                    P[j] = 0.5*mZ
                output.write(f"mass = ops.nodeMass(ndI)\n")
                output.write(f"mass2 = np.multiply({P},Lele)\n")
                output.write(f"ops.mass(ndI,*(np.add(mass,mass2)))\n")
                output.write(f"ops.mass(ndJ,*(np.add(mass,mass2)))\n")                
                    
            # Point masses
            pmInfo = inputt.readline().split()
            Npointmass = int(pmInfo[1])
            for i in range(Npointmass):
                pmInfo = inputt.readline().split()
                nd = int(pmInfo[0])
                P = [0]*ndf
                for j in range(ndf):
                    P[j] = float(pmInfo[j+1])
                output.write(f'ops.mass({nd},*{P})\n')

            output.write('ops.analyze(1)\n')
            output.write('ops.reactions()\n')
            output.write('print(ops.eigen(100))\n')
            output.write(f"ops.remove('loadPattern',{tag})\n")

            output.write(f"output.write('LOAD CASE {tag}\\n')\n")

            output.write(f'ndm={ndm}\n')
            output.write(f'ndf={ndf}\n')        
            output.write("exec(open('writeOpenSeesResults.py').read())\n")
            output.write('\n')

            output.write("output.write('\\n')\n")
            
    output.write('output.close()\n')
    output.close()
    output2.close()
    inputt.close()



import numpy as np
import math

def barSizes():
    """
    Returns a list of US bar designations 3--18
    """
    return [3,4,5,6,7,8,9,10,11,14,18]


def barNums():
    """
    Returns a list of US bar designations 3--18
    """
    return barSizes()


def barArea(designation):
    """
    Cross-sectional area (in^2) of round bars
    US designations 3,4,5,6,7,8,9,10,11,14,18

    designation (integer, 3--18)
    """
    
    if designation not in barSizes():
        return None
    
    return {
        3: 0.11,
        4: 0.20,
        5: 0.31,
        6: 0.44,
        7: 0.60,
        8: 0.79,
        9: 1.00,
        10: 1.27,
        11: 1.56,
        14: 2.25,
        18: 4.00
    }[designation]


def barDiam(designation):
    """
    Diameter (in) of round bars
    US designations 3,4,5,6,7,8,9,10,11,14,18

    designation (integer, 3--18)
    """
    
    if designation not in barSizes():
        return None
    
    return {
        3: 0.375,
        4: 0.50,
        5: 0.625,
        6: 0.75,
        7: 0.875,
        8: 1.00,
        9: 1.128,
        10: 1.27,
        11: 1.41,
        14: 1.693,
        18: 2.257
    }[designation]


def selectBar(Abreq):
    """
    Returns the smallest round bar with required area
    US designations 3--18

    Abreq - required bar area (in^2)
    """
    
    for bar in barSizes():
        if Abreq <= barArea(bar):
            return bar

    return None

def straightAnchorage(fc, fy, bar, psit=1.0, psie=1.0, lam=1.0):
    """
    Returns required development length (in) for straight anchorage

    fc - concrete compressive strength (psi)
    fy - bar strength (psi)
    bar - bar size, US designation 3--18
    psit - top steel factor (optional, default = 1)
    psie - epoxy factor (optional, default = 1)
    lam - lightweight concrete factor (optional, default = 1)
    """

    if bar not in barSizes():
        return None
    
    db = barDiam(bar)

    if bar <= 6:
        ld = db*(fy*psit*psie)/(25*lam*fc**0.5)
    else:
        ld = db*(fy*psit*psie)/(20*lam*fc**0.5)        

    return ld

def hookedAnchorage(fc, fy, bar, psie=1.0, psic=1.0, psir=1.0, lam=1.0):
    """
    Returns required development length (in) for hooked anchorage

    fc - concrete compressive strength (psi)
    fy - bar strength (psi)
    bar - bar size, US designation 3--18
    psie - epoxy factor (optional, default = 1)
    psic - (optional, default = 1)
    psir - (optional, default = 1)
    lam - lightweight concrete factor (optional, default = 1)
    """
    
    db = barDiam(bar)

    ldh = db*(fy*psie*psic*psir)/(50*lam*fc**0.5)

    return ldh

def beta1(fc):
    """
    Returns WSB parameter, $\beta_1$

    fc - concrete compressive strength (psi)
    """
    
    if fc < 4000:
        return 0.85
    if fc > 8000:
        return 0.65
    return 0.85-0.05*(fc-4000)/1000.0

def beamPhi(et):
    """
    Returns strength reduction factor, $\phi$, for flexural members
    with non-conforming transverse reinforcement

    et - strain in tension steel at ultimate strength
    """
    
    if et < 0.002:
        return 0.65
    if et > 0.005:
        return 0.90
    return 0.65 + 250.0/3*(et-0.002)

def beamPhiRound(et):
    """
    Returns strength reduction factor, $\phi$, for flexural members
    with conforming spiral transverse reinforcement

    et - strain in tension steel at ultimate strength
    """    
    if et < 0.002:
        return 0.75
    if et > 0.005:
        return 0.90
    return 0.75 + 150.0/3*(et-0.002)

def flexuralStrength(fc, fy, E, b, As, d, Asp=0, dp=0):
    """
    Calculate the flexural strength (nominal and avaialble)
    for a rectangular RC section with compression steel
    
    OUTPUTS [Mn,phi,c]
    Mn - nominal flexural strength (lb-in)
    phi - strength reduction factor
    c - depth from compression face to neutral axis (in)

    INPUTS
    fc - concrete compressive strength (psi)
    fy - steel strength (psi)
    E - steel modulus of elasticity (psi)
    b - section width (in)
    As - total area of tension steel (in^2)
    d - depth from compression face to centroid of tension steel (in)
    Asp - total area of comrpession steel (in^2) (optional, default=0)
    dp - depth from compression face to centroid of compression steel (in) (optional, default=0)
    """

    # Initial guess for NA
    c = d/4.0

    b1 = beta1(fc)

    T = As*fy
    a = b1*c
    Cc = 0.85*fc*a*b
    Cs = 0.0

    i = 0
    iMax = int(d/0.001)
    while i < iMax and abs(T-(Cc+Cs)) > 0.001*T:
        i = i+1

        if T > (Cc+Cs):
            c = c + 0.001
        else:
            c = c - 0.001

        # Compression concrete
        a = b1*c
        Cc = 0.85*fc*a*b

        # Compression steel
        esp = (c-dp)/c*0.003
        fsp = abs(E*esp)
        if fsp > fy:
            fsp = fy
        if dp < a:
            fsp = fsp-0.85*fc
        Cs = Asp*fsp

        # Tension steel
        es = (d-c)/c*0.003
        fs = abs(E*es)
        if fs > fy:
            fs = fy
        T = As*fs

    es = (d-c)/c*0.003
    phi = beamPhi(es)

    Mn = Cc*(d-0.5*a) + Cs*(d-dp)

    return [phi,Mn,c]
    

def designStrength(fc, fy, E, b, h, Nlayers, d, As, epst):
    """
    Calculate the available design strength
    for a rectangular RC section with multiple layers of steel
    
    OUTPUTS [Pn,Mn,phiPn,phiMn]
    Pn - nominal axial strength (lb)
    phiPn - available axial strength (lb)
    Mn - nominal flexural strength (lb-in)
    phiMn - available flexural strength (lb-in)

    INPUTS
    fc - concrete compressive strength (psi)
    fy - steel strength (psi)
    E - steel modulus of elasticity (psi)
    b - section width (in)
    h - section depth (in)
    Nlayers - number of steel layers
    d[] - list of steel layer depths (in) (index 0 is extreme tension layer)
    As[] - list of steel layer areas (total steel in layer) (in^2) (index 0 is extreme tension layer)
    epst - strain in extreme tension steel (negative compression, positive tension)
    """

    b1 = beta1(fc)

    c = 0.003/(0.003+epst)*d[0]
    a = min(h,b1*c)
    Cc = 0.85*fc*a*b
        
    Pn = Cc
    Mn = Cc*(0.5*h-0.5*a)

    for i in range(Nlayers):
        eps = (c-d[i])/c*-0.003
        fs = min(fy,math.fabs(eps*E))
        if d[i] < a:
            fs = fs - 0.85*fc
        Fs = np.sign(eps)*fs*As[i]
        Pn = Pn - Fs
        Mn = Mn + Fs*(d[i]-0.5*h)
        
    phi = beamPhi(epst)

    phiPn = phi*Pn
    phiMn = phi*Mn

    return [Pn,Mn,phiPn,phiMn]


def interactionDiagram(fc, fy, E, b, h, rhog, Nbars, gamma, Npts=100):
    """
    Calculate the interaction diagram for a rectangular RC section
    
    OUTPUTS
    [Pn/Ag,Mn/(Ag*h),phiPn/Ag,phiMn/(Ag*h),Pn,Mn,phiPn,phiMn]
    Pn - nominal axial strength (lb)
    phiPn - available axial strength (lb)
    Mn - nominal flexural strength (lb-in)
    phiMn - available flexural strength (lb-in)
    Ag - gross section area (b*h)
    
    INPUTS
    fc - concrete compressive strength (psi)
    fy - steel strength (psi)
    E - steel modulus of elasticity (psi)
    b - section width (in)
    h - section depth (in)
    rhog - gross reinforcing ratio, e.g., 0.02 for 2%
    Nbars - number of bars on any *one* side of the section
    gamma - ratio of steel moment arm to section depth, h
    """

    if Npts < 2:
        Npts = 100
    output = []

    b1 = beta1(fc)

    Ag = b*h
    Ast = rhog*Ag

    d = [0]*Nbars
    As = [0]*Nbars

    As[0] = Ast*Nbars/(2*Nbars+2*(Nbars-2))
    d[0] = h-0.5*(h-gamma*h)
    for i in range(1,Nbars-1):
        As[i] = Ast*2/(2*Nbars+2*(Nbars-2))
        d[i] = d[Nbars-1] + (gamma*h)*i/(Nbars-1)
    As[Nbars-1] = As[0]
    d[Nbars-1] = 0.5*(h-gamma*h)
    
    Pno = 0.8*(0.85*fc*(Ag-Ast) + fy*Ast)
    phi = beamPhi(0)
    phiPn = phi*Pno

    phiPnT = -beamPhi(0.1)*fy*Ast

    output[0,:] = [Pno/Ag,0.0,phiPn/Ag,0.0,Pno,0.0,phiPn,0.0]

    epst = -0.003 + 1e-16

    start = 2*h
    stop = 0.5*d[Nbars-1]
    step = (stop-start)/Npts

    ii = 1
    for c in np.arange(start,stop,step):
        a = min(h,b1*c)
        Cc = 0.85*fc*a*b
        
        Pn = Cc
        Mn = Cc*(0.5*h-0.5*a)

        for i in range(Nbars):
            eps = (c-d[i])/c*-0.003
            fs = min(fy,math.fabs(eps*E))
            if d[i] < a:
                fs = fs - 0.85*fc
            Fs = np.sign(eps)*fs*As[i]
            Pn = Pn - Fs
            Mn = Mn + Fs*(d[i]-0.5*h)
        
        if Pn > Pno:
            Pn = Pno
        
        eps1 = (c-d[0])/c*-0.003
        phi = beamPhi(eps1)

        phiPn = phi*Pn
        phiMn = phi*Mn

        output[ii,:] = [Pn/Ag,Mn/(Ag*h),phiPn/Ag,phiMn/(Ag*h),Pn,Mn,phiPn,phiMn]
        ii = ii+1

    PnT = -fy*Ast
    phiPnT = beamPhi(0.1)*PnT

    output[Npts+1,:] = [PnT/Ag,0.0,phiPnT/Ag,0.0,PnT,0.0,phiPnT,0.0]

    return output


def interactionDiagramRound(fc, fy, E, h, rhog, Nbars, gamma, Npts=100):
    """
    Calculate the interaction diagram for a circular RC section with
    conforming spiral transverse reinforcement
    
    OUTPUTS
    [Pn/Ag,Mn/(Ag*h),phiPn/Ag,phiMn/(Ag*h),Pn,Mn,phiPn,phiMn]
    Pn - nominal axial strength (lb)
    phiPn - available axial strength (lb)
    Mn - nominal flexural strength (lb-in)
    phiMn - available flexural strength (lb-in)
    Ag - gross section area (pi*h*h/4)
    
    INPUTS
    fc - concrete compressive strength (psi)
    fy - steel strength (psi)
    E - steel modulus of elasticity (psi)
    h - section diameter (in)
    rhog - gross reinforcing ratio, e.g., 0.02 for 2%
    Nbars - total number of bars in the section
    gamma - ratio of steel moment arm to section diameter, h
    """

    if Npts < 2:
        Npts = 100
    output = []
    
    b1 = beta1(fc)

    Ag = h*h*3.14159/4.0
    Ast = rhog*Ag

    d = [0]*(Nbars//2)
    As = [0]*(Nbars//2)

    for i in range(Nbars//2):
        As[i] = 2*Ast/Nbars
        theta = (i+0.5)*2*3.14159/Nbars
        d[i] = 0.5*h - 0.5*gamma*h*math.cos(theta)
    
    Pno = 0.85*(0.85*fc*(Ag-Ast) + fy*Ast)
    phi = beamPhiRound(0)
    phiPn = phi*Pno

    output[0,:] = [Pno/Ag,0.0,phiPn/Ag,0.0,Pno,0.0,phiPn,0.0]
    
    epst = -0.003 + 1e-16

    start = 2*h
    stop = 0.5*d[0]
    step = (stop-start)/Npts
    
    ii = 1    
    for c in np.arange(start,stop,step):
        a = min(h,b1*c)
        if a <= 0.5*h:
            theta = math.acos(1.0-a/(0.5*h))
        else:
            theta = 3.14159 - math.acos(a/(0.5*h)-1.0)

        Ac = h*h*(theta-math.sin(theta)*math.cos(theta))/4.0
        Cc = 0.85*fc*Ac
        Pn = Cc

        ybar = h*h*h/12.0*(math.sin(theta))**3/Ac
        
        Mn = Cc*ybar

        for i in range(Nbars//2):
            eps = (c-d[i])/c*-0.003
            fs = min(fy,math.fabs(eps*E))
            if d[i] < a:
                fs = fs - 0.85*fc
            Fs = np.sign(eps)*fs*As[i]
            Pn = Pn - Fs
            Mn = Mn + Fs*(d[i]-0.5*h)
        
        if Pn > Pno:
            Pn = Pno
        
        phi = beamPhiRound(epst)

        phiPn = phi*Pn
        phiMn = phi*Mn

        output[ii,:] = [Pn/Ag,Mn/(Ag*h),phiPn/Ag,phiMn/(Ag*h),Pn,Mn,phiPn,phiMn]
        ii = ii+1        

        epst = epst + 0.00005

    PnT = -fy*Ast
    phiPnT = beamPhi(0.1)*PnT

    output[Npts+1,:] = [PnT/Ag,0.0,phiPnT/Ag,0.0,PnT,0.0,phiPnT,0.0]

    return output



import requests

def GetPaginatedList(url, accessToken, payload={}):
        """
        Returns a full list of Canvas items

        url - Canvas URL to access (string)
        accessToken - Canvas API access token (string)
        payload - payload dictionary for the Canvas API request (optional, default={})
        """
        
        authHeader = {'Authorization': 'Bearer ' + accessToken}
        
        itemList = []
        
        r = requests.get(url, headers=authHeader, data=payload)
        for item in r.json():
                itemList.append(item)
        
        while 'next' in r.links:
                r = requests.get(r.links['next']['url'], headers=authHeader, data=payload)
                for item in r.json():
                        itemList.append(item)
        
        return itemList


def UploadFile(url, accessToken, fileName, displayName=None):
        """
        Upload a file to Canvas

        url - Canvas URL to which file will be posted (string)
        accessToken - Canvas API access token (string)
        fileName - local file to upload (string)
        displayName - file name to show on Canvas (optional, default=fileName)
        """

        authHeader = {'Authorization': 'Bearer ' + accessToken}
        
        # Step 1
        payload = {}
        if displayName is not None: 
                payload = {'name': displayName}
        results = requests.post(url, headers=authHeader, data=payload)
        abc = results.json()
        
        # Step 2
        results = requests.post(abc['upload_url'], data=abc['upload_params'],  allow_redirects=False, files={'file': open(fileName,'rb')})
        abc = results.headers
        
        # Step 3
        results = requests.post(abc['location'], headers=authHeader)
        abc = results.json()
        
        if abc['upload_status'] != 'success':
                return -1
        return 0

def GetFolderID(courseURL, accessToken, folderName, makeFolder = True):
        """
        Find a folder ID within a course based on folder name

        courseURL - Canvas URL to course where folder is located (string)
        accessToken - Canvas API access token (string)
        folderName - name of folder to search (string)
        makeFolder - create the folder if not found (optional, default=True)

        Searches '{courseURL}/folders/by_path/{folderName}'
        """

        authHeader = {'Authorization': 'Bearer ' + accessToken}

        
        r = requests.get(courseURL + f'folders/by_path/{folderName}', headers=authHeader)
        if r.status_code == requests.codes.ok:
                folderList = r.json()
                folderID = folderList[-1]['id']
        elif makeFolder:
                payload = {'name': folderName, 'parent_folder_path': '/'}
                r = requests.post(courseURL + 'folders', headers=authHeader, data=payload)
                print(f'Creating folder {folderName}')
                folderInfo = r.json()
                folderID = folderInfo['id']

        return folderID



def MyRound(x, base=10):
        x = int(base*round(float(x)/base))
        return float(x)

def S4SWidth(bnom):
        """
        Return width based on nominal dimension of S4S lumber

        bnom - nominal width (inch)
        """
        
        if bnom == 1:
                return 0.75
        else:
                return bnom-0.5

def S4SDepth(dnom):
        """
        Return depth based on nominal dimension of S4S lumber

        dnom - nominal depth (inch)
        """
        
        if dnom < 8:
                return dnom-0.5
        else:
                return dnom-0.75

def S4SArea(bnom, dnom):
        """
        Return section area based on nominal dimensions of S4S lumber

        bnom - nominal width (inch)
        dnom - nominal depth (inch)
        """
        
        b = S4SWidth(bnom)
        d = S4SDepth(dnom)

        return b*d

def S4SModulus(bnom, dnom):
        """
        Return section modulus, S, based on nominal dimensions of S4S lumber

        bnom - nominal width (inch)
        dnom - nominal depth (inch)
        """
        
        b = S4SWidth(bnom)
        d = S4SDepth(dnom)
        
        return b*d**2/6

def TimberWidth(bnom):
        """
        Return width based on nominal dimension of timber

        bnom - nominal width (inch)
        """
        
        return bnom-0.5

def TimberDepth(dnom):
        """
        Return depth based on nominal dimension of timber

        dnom - nominal width (inch)
        """
        
        return dnom-0.5

def TimberArea(bnom, dnom):
        """
        Return section area based on nominal dimensions of timber

        bnom - nominal width (inch)
        dnom - nominal depth (inch)
        """
        
        b = TimberWidth(bnom)
        d = TimberDepth(dnom)
        
        return b*d

def TimberModulus(bnom, dnom):
        """
        Return section modulus, S, based on nominal dimensions of timber

        bnom - nominal width (inch)
        dnom - nominal depth (inch)
        """
        
        b = TimberWidth(bnom)
        d = TimberDepth(dnom)
        
        return b*d**2/6

def StabilityFactor(FcE, Fcstar, c=0.8):
        """
        Return stability factor, C_P, based on Euler and crushing stresses

        FcE - Euler buckling stress
        Fcstar - Crushing stress
        c - member factor (optional, default = 0.8)
            Note: use c = 0.9 for glulam
        """
        
        a = FcE/Fcstar
        b = (1+a)/(2*c)

        return b - (b**2-a/c)**0.5

def IncisingFactor(type):
        if type == 'Fb' or type == 'Ft' or type == 'Fc' or type == 'Fv':
                return 0.8
        if type == 'Fcp':
                return 1.0
        if type == 'E' or type == 'Emin':
                return 0.95
        return 1.0

def WetServiceFactor(type, b=2, d=2):
        if b > 4 and d > 4:
                # Timbers
                if type == 'Fcp':
                        return 0.67
                if type == 'Fc':
                        return 0.91
                return 1.0

        if type == 'Fb':
                return 0.85
        if type == 'Ft':
                return 1.0
        if type == 'Fv':
                return 0.97
        if type == 'Fcp':
                return 0.67
        if type == 'Fc':
                return 0.8
        if type == 'E' or type == 'Emin':
                return 0.9
        if type == 'Z':
                return 0.7
        return 1.0

def TemperatureFactor(type,T,WetOrDry='Dry'):
	if T <= 100:
		return 1.0

	if type == 'Fb' or type == 'Fv' or type == 'Fc' or type == 'Fcp':
		if T > 100 and T <= 125:
			if WetOrDry == 'Wet':
				return 0.7
			else:
				return 0.8
		if T > 125 and T <= 150:
			if WetOrDry == 'Wet':
				return 0.5
			else:
				return 0.7

	if type == 'Ft' or type == 'E' or type == 'Emin':
		if T > 100:
			return 0.9

def VolumeFactor(L, d, b, x=0.1):
        """
        Return volume factor, C_V, for glulam members

        L - member length (ft)
        d - member depth (inch)
        b - member width (inch)
        x - factor for glulam species (optional, default = 0.1)
            Note: use x = 0.05 for Southern Pines
        """
        
        CV = (21/L)**x * (12/d)**x * (5.125/b)**x

        return min(1.0,CV)

def SizeFactor(type, b, d):
        if type == 'Fb':
		# Timbers
                if b > 4 and d <= 12:
                        return 1.0
                if b > 4 and d > 12:
                        return (12.0/d)**(1./9)
                        
                if d <= 4:
                        return 1.5
                if d == 5:
                        return 1.4
                if d == 6:
                        return 1.3   
                        
                if d == 8 and b == 4:
                        return 1.3
                if d == 8 and b < 4:
                        return 1.2
        
                if d == 10 and b == 4:
                        return 1.2
                if d == 10 and b < 4:
                        return 1.1

                if d == 12 and b == 4:
                        return 1.1
                if d == 12 and b < 4:
                        return 1.0   
                        
                if d >= 14 and b == 4:
                        return 1.0
                if d >= 14 and b < 4:
                        return 0.9   
                        
                return 1.0

        if type == 'Ft':
		# Timbers
                if b > 4:
                        return 1.0

                if d <= 4:
                        return 1.5
                if d == 5:
                        return 1.4
                if d == 6:
                        return 1.3
                if d == 8:
                        return 1.2
                if d == 10:
                        return 1.1
                if d == 12:
                        return 1.0
                if d >= 14:
                        return 0.9

        if type == 'Fc':
                # Timbers
                if b > 4:
                        return 1.0

                if d <= 4:
                        return 1.15
                if d == 5:
                        return 1.1
                if d == 6:
                        return 1.1
                if d == 8:
                        return 1.05
                if d == 10:
                        return 1.0
                if d == 12:
                        return 1.0
                if d >= 14:
                        return 0.9
                        
        return 1.0      


 
def FormatConversionFactor(type):    
        if type == 'Fb':
                return 2.54
        if type == 'Ft':  
                return 2.70
        if type == 'Fv':  
                return 2.88
        if type == 'Fc':  
                return 2.40
        if type == 'Fcp': 
                return 1.67
        if type == 'Emin': 
                return 1.76
        if type == 'Z' or type == 'W':
                return 3.32
        return 1.0
                        
def phiFactor(type):
        if type == 'Fb':
                return 0.85
        if type == 'Ft':
                return 0.80
        if type == 'Fv':   
                return 0.75
        if type == 'Fc':   
                return 0.90
        if type == 'Fcp':  
                return 0.90
        if type == 'Emin': 
                return 0.85
        if type == 'Z' or type == 'W':
                return 0.65
        return 1.0

def GroupActionFactor(Am,As,n,D=1.0,s=4,Em=1.4e6,Es=1.4e6):
        """
        Return group action factor for a bolt group

        Am - area of main member (inch^2)
        As - area of side member (inch^2)
        n - number of rows
        D - bolt diameter (inch, optional, default = 1.0)
        s - bolt spacing (inch, optional, default = 4.0)
        Em - elastic modulus of main member (psi, optional, default = 1.4e6)
        Es - elastic modulus of side member (psi, optional, default = 1.4e6)
        """
        
        if D < 0.25:
                return 1.0

        REA = min(Es*As/(Em*Am),Em*Am/(Es*As))
        gamma = 180000*D**1.5
        u = 1 + gamma*s/2*(1/(Em*Am)+1/(Es*As))
        m = u - (u**2-1)**0.5
        num = m*(1-m**(2*n))
        den = n*((1+REA*m**n)*(1+m)-1+m**(2*n))
        Cg = (num/den)*(1+REA)/(1-m)
        
        return Cg

def BoltSingleShear(tm,ts,D,G):
        """
        Return bolt strength in single shear
        
        tm - thickness of main member (inch)
        ts - thinkness of side member (inch)
        D - bolt diameter (inch)
        G - specific gravity of members

        Returns list of four values
        [Z||,Zpside,Zpmain,Zperp]
          loading parallel to grain of both members
          loading perpendicular to grain side member
          loading perpendicular to grain main member
          loading perpendicular to grain of both members
        """
        
        lm = tm
        ls = ts
        Rt = lm/ls

        Zref = [0,0,0,0]

        Fyb = 45000.0
        if D < 0.25:
                Fyb = 100000.0

	# Z parallel
        theta = 0
        Fem = MyRound(11200.0*G,50)
        Fes = MyRound(11200.0*G,50)

        Re = Fem/Fes

        Ktheta = 1 + 0.25*(theta/90.0)
        Rd = 4*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZIm = D*lm*Fem/Rd
        Z = ZIm
        ZIs = D*ls*Fes/Rd
        Z = min(Z,ZIs)
        k1 = (math.sqrt(Re+2*Re**2*(1+Rt+Rt**2)+Rt**2*Re**3)-Re*(1+Rt))/(1+Re)
        Rd = 3.6*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZII = k1*D*ls*Fes/Rd
        Z = min(Z,ZII)
        Rd = 3.2*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        k2 = -1+math.sqrt(2*(1+Re)+(2*Fyb*(1+2*Re)*D**2)/(3*Fem*lm**2))
        ZIIIm = k2*D*lm*Fem/((1+2*Re)*Rd)
        Z = min(Z,ZIIIm)
        k3 = -1+math.sqrt(2*(1+Re)/Re+(2*Fyb*(2+Re)*D**2)/(3*Fem*ls**2))
        ZIIIs = k3*D*ls*Fem/((2+Re)*Rd)
        Z = min(Z,ZIIIs)
        ZIV = D**2/Rd*math.sqrt(2*Fem*Fyb/(3*(1+Re)))
        Z = min(Z,ZIV)
        Zref[0] = MyRound(Z,1)

        # Z perpendicular
        theta = 90
        Fem = MyRound(6100.0*G**1.45/D**0.5,50)
        Fes = MyRound(6100.0*G**1.45/D**0.5,50)

        Re = Fem/Fes

        Ktheta = 1 + 0.25*(theta/90.0)
        Rd = 4*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZIm = D*lm*Fem/Rd
        Z = ZIm
        ZIs = D*ls*Fes/Rd
        Z = min(Z,ZIs)
        k1 = (math.sqrt(Re+2*Re**2*(1+Rt+Rt**2)+Rt**2*Re**3)-Re*(1+Rt))/(1+Re)
        Rd = 3.6*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZII = k1*D*ls*Fes/Rd
        Z = min(Z,ZII)
        Rd = 3.2*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        k2 = -1+math.sqrt(2*(1+Re)+(2*Fyb*(1+2*Re)*D**2)/(3*Fem*lm**2))
        ZIIIm = k2*D*lm*Fem/((1+2*Re)*Rd)
        Z = min(Z,ZIIIm)
        k3 = -1+math.sqrt(2*(1+Re)/Re+(2*Fyb*(2+Re)*D**2)/(3*Fem*ls**2))
        ZIIIs = k3*D*ls*Fem/((2+Re)*Rd)
        Z = min(Z,ZIIIs)
        ZIV = D**2/Rd*math.sqrt(2*Fem*Fyb/(3*(1+Re)))
        Z = min(Z,ZIV)
        Zref[3] = MyRound(Z,1)

        # Z perpendicular to main
        theta = 90
        Fem = MyRound(6100.0*G**1.45/D**0.5,50)
        Fes = MyRound(11200.0*G,50)

        Re = Fem/Fes

        Ktheta = 1 + 0.25*(theta/90.0)
        Rd = 4*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZIm = D*lm*Fem/Rd
        Z = ZIm
        ZIs = D*ls*Fes/Rd
        Z = min(Z,ZIs)
        k1 = (math.sqrt(Re+2*Re**2*(1+Rt+Rt**2)+Rt**2*Re**3)-Re*(1+Rt))/(1+Re)
        Rd = 3.6*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZII = k1*D*ls*Fes/Rd
        Z = min(Z,ZII)
        Rd = 3.2*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        k2 = -1+math.sqrt(2*(1+Re)+(2*Fyb*(1+2*Re)*D**2)/(3*Fem*lm**2))
        ZIIIm = k2*D*lm*Fem/((1+2*Re)*Rd)
        Z = min(Z,ZIIIm)
        k3 = -1+math.sqrt(2*(1+Re)/Re+(2*Fyb*(2+Re)*D**2)/(3*Fem*ls**2))
        ZIIIs = k3*D*ls*Fem/((2+Re)*Rd)
        Z = min(Z,ZIIIs)
        ZIV = D**2/Rd*math.sqrt(2*Fem*Fyb/(3*(1+Re)))
        Z = min(Z,ZIV)
        Zref[2] = MyRound(Z,1)

        # Z perpendicular to side member
        theta = 90
        Fem = MyRound(11200.0*G,50)
        Fes = MyRound(6100.0*G**1.45/D**0.5,50)

        Re = Fem/Fes

        Ktheta = 1 + 0.25*(theta/90.0)
        Rd = 4*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZIm = D*lm*Fem/Rd
        Z = ZIm
        ZIs = D*ls*Fes/Rd
        Z = min(Z,ZIs)
        k1 = (math.sqrt(Re+2*Re**2*(1+Rt+Rt**2)+Rt**2*Re**3)-Re*(1+Rt))/(1+Re)
        Rd = 3.6*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZII = k1*D*ls*Fes/Rd
        Z = min(Z,ZII)
        Rd = 3.2*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        k2 = -1+math.sqrt(2*(1+Re)+(2*Fyb*(1+2*Re)*D**2)/(3*Fem*lm**2))
        ZIIIm = k2*D*lm*Fem/((1+2*Re)*Rd)
        Z = min(Z,ZIIIm)
        k3 = -1+math.sqrt(2*(1+Re)/Re+(2*Fyb*(2+Re)*D**2)/(3*Fem*ls**2))
        ZIIIs = k3*D*ls*Fem/((2+Re)*Rd)
        Z = min(Z,ZIIIs)
        ZIV = D**2/Rd*math.sqrt(2*Fem*Fyb/(3*(1+Re)))
        Z = min(Z,ZIV)
        Zref[1] = MyRound(Z,1)

        return Zref


def BoltDoubleShear(tm,ts,D,G):
        """
        Return bolt strength in double shear
        
        tm - thickness of main member (inch)
        ts - thinkness of side member (inch)
        D - bolt diameter (inch)
        G - specific gravity of members

        Returns list of four values
        [Z||,Zpside,Zpmain,Zperp]
          loading parallel to grain of both members
          loading perpendicular to grain side member
          loading perpendicular to grain main member
          loading perpendicular to grain of both members
        """
        
        lm = tm
        ls = ts
        Rt = lm/ls

        Fyb = 45000.0
        if D < 0.25:
                Fyb = 100000.0

        Zref = [0,0,0]

        # Z parallel
        theta = 0
        Fem = MyRound(11200.0*G,50)
        Fes = MyRound(11200.0*G,50)

        Re = Fem/Fes

        Ktheta = 1 + 0.25*(theta/90.0)
        Rd = 4*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZIm = D*lm*Fem/Rd
        Z = ZIm
        ZIs = 2.0*D*ls*Fes/Rd
        Z = min(Z,ZIs)
        Rd = 3.2*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        k3 = -1+math.sqrt(2*(1+Re)/Re+(2*Fyb*(2+Re)*D**2)/(3*Fem*ls**2))
        ZIIIs = 2.0*k3*D*ls*Fem/((2+Re)*Rd)
        Z = min(Z,ZIIIs)
        ZIV = 2.0*D**2/Rd*math.sqrt(2*Fem*Fyb/(3*(1+Re)))
        Z = min(Z,ZIV)
        Zref[0] = MyRound(Z,10)

        # Z perpendicular to main
        theta = 90
        Fem = MyRound(6100.0*G**1.45/D**0.5,50)
        Fes = MyRound(11200.0*G,50)

        Re = Fem/Fes

        Ktheta = 1 + 0.25*(theta/90.0)
        Rd = 4*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZIm = D*lm*Fem/Rd
        Z = ZIm
        ZIs = 2.0*D*ls*Fes/Rd
        Z = min(Z,ZIs)
        Rd = 3.2*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        k3 = -1+math.sqrt(2*(1+Re)/Re+(2*Fyb*(2+Re)*D**2)/(3*Fem*ls**2))
        ZIIIs = 2.0*k3*D*ls*Fem/((2+Re)*Rd)
        Z = min(Z,ZIIIs)
        ZIV = 2.0*D**2/Rd*math.sqrt(2*Fem*Fyb/(3*(1+Re)))
        Z = min(Z,ZIV)
        Zref[2] = MyRound(Z,10)

        # Z perpendicular to side member
        theta = 90
        Fem = MyRound(11200.0*G,50)
        Fes = MyRound(6100.0*G**1.45/D**0.5,50)

        Re = Fem/Fes

        Ktheta = 1 + 0.25*(theta/90.0)
        Rd = 4*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        ZIm = D*lm*Fem/Rd
        Z = ZIm
        ZIs = 2.0*D*ls*Fes/Rd
        Z = min(Z,ZIs)
        Rd = 3.2*Ktheta
        if D < 0.25:
                Rd = max(2.2,10*D + 0.5)
        k3 = -1+math.sqrt(2*(1+Re)/Re+(2*Fyb*(2+Re)*D**2)/(3*Fem*ls**2))
        ZIIIs = 2.0*k3*D*ls*Fem/((2+Re)*Rd)
        Z = min(Z,ZIIIs)
        ZIV = 2.0*D**2/Rd*math.sqrt(2*Fem*Fyb/(3*(1+Re)))
        Z = min(Z,ZIV)
        Zref[1] = MyRound(Z,10)

        return Zref

